{
    "$graph": [
        {
            "class": "Workflow",
            "label": "Detect Docm variants",
            "requirements": [
                {
                    "class": "SubworkflowFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ],
                    "id": "#main/docm_vcf"
                },
                {
                    "type": "boolean",
                    "id": "#main/filter_docm_variants"
                },
                {
                    "type": "File",
                    "id": "#main/interval_list"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#main/normal_bam"
                },
                {
                    "type": [
                        "string",
                        "File"
                    ],
                    "secondaryFiles": [
                        ".fai",
                        "^.dict"
                    ],
                    "id": "#main/reference"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#main/tumor_bam"
                }
            ],
            "steps": [
                {
                    "run": "#docm_gatk_haplotype_caller.cwl",
                    "in": [
                        {
                            "source": "#main/tumor_bam",
                            "id": "#main/GATK_haplotype_caller/bam"
                        },
                        {
                            "source": "#main/docm_vcf",
                            "id": "#main/GATK_haplotype_caller/docm_vcf"
                        },
                        {
                            "source": "#main/interval_list",
                            "id": "#main/GATK_haplotype_caller/interval_list"
                        },
                        {
                            "source": "#main/normal_bam",
                            "id": "#main/GATK_haplotype_caller/normal_bam"
                        },
                        {
                            "source": "#main/reference",
                            "id": "#main/GATK_haplotype_caller/reference"
                        }
                    ],
                    "out": [
                        "#main/GATK_haplotype_caller/docm_raw_variants"
                    ],
                    "id": "#main/GATK_haplotype_caller"
                },
                {
                    "run": "#bgzip.cwl",
                    "in": [
                        {
                            "source": "#main/GATK_haplotype_caller/docm_raw_variants",
                            "id": "#main/bgzip/file"
                        }
                    ],
                    "out": [
                        "#main/bgzip/bgzipped_file"
                    ],
                    "id": "#main/bgzip"
                },
                {
                    "run": "#bgzip.cwl",
                    "in": [
                        {
                            "source": "#main/docm_filter/docm_filtered_variants",
                            "id": "#main/bgzip2/file"
                        }
                    ],
                    "out": [
                        "#main/bgzip2/bgzipped_file"
                    ],
                    "id": "#main/bgzip2"
                },
                {
                    "run": "#vt_decompose.cwl",
                    "in": [
                        {
                            "source": "#main/index/indexed_vcf",
                            "id": "#main/decompose/vcf"
                        }
                    ],
                    "out": [
                        "#main/decompose/decomposed_vcf"
                    ],
                    "id": "#main/decompose"
                },
                {
                    "run": "#filter_vcf_docm.cwl",
                    "in": [
                        {
                            "source": "#main/decompose/decomposed_vcf",
                            "id": "#main/docm_filter/docm_raw_variants"
                        },
                        {
                            "source": "#main/filter_docm_variants",
                            "id": "#main/docm_filter/filter_docm_variants"
                        },
                        {
                            "source": "#main/normal_bam",
                            "id": "#main/docm_filter/normal_bam"
                        },
                        {
                            "source": "#main/tumor_bam",
                            "id": "#main/docm_filter/tumor_bam"
                        }
                    ],
                    "out": [
                        "#main/docm_filter/docm_filtered_variants"
                    ],
                    "id": "#main/docm_filter"
                },
                {
                    "run": "#index_vcf.cwl",
                    "in": [
                        {
                            "source": "#main/bgzip/bgzipped_file",
                            "id": "#main/index/vcf"
                        }
                    ],
                    "out": [
                        "#main/index/indexed_vcf"
                    ],
                    "id": "#main/index"
                },
                {
                    "run": "#index_vcf.cwl",
                    "in": [
                        {
                            "source": "#main/bgzip2/bgzipped_file",
                            "id": "#main/index2/vcf"
                        }
                    ],
                    "out": [
                        "#main/index2/indexed_vcf"
                    ],
                    "id": "#main/index2"
                }
            ],
            "id": "#main",
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#main/index2/indexed_vcf",
                    "secondaryFiles": [
                        ".tbi"
                    ],
                    "id": "#main/docm_variants_vcf"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "label": "bgzip VCF",
            "baseCommand": [
                "/usr/local/bin/bgzip"
            ],
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/samtools:1.11--h6270b1f_0"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 4000
                }
            ],
            "stdout": "$(inputs.file.basename).gz",
            "arguments": [
                "-c"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#bgzip.cwl/file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "id": "#bgzip.cwl/bgzipped_file",
                    "outputBinding": {
                        "glob": "$(inputs.file.basename).gz"
                    }
                }
            ],
            "id": "#bgzip.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "HaplotypeCaller (GATK4)",
            "baseCommand": [
                "/bin/bash",
                "docm_haplotypeCaller.sh"
            ],
            "requirements": [
                {
                    "class": "ResourceRequirement",
                    "ramMin": 9000
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "broadinstitute/gatk:4.1.2.0"
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entryname": "docm_haplotypeCaller.sh",
                            "entry": "set -o pipefail\nset -o errexit\n\n# Running haplotype caller using the newly created interval list\nif [[ \"$#\" == 5 ]];then # If normal_bam is passed.\n    # explicitly capturing variables\n    reference=$1\n    normal_bam=$2\n    tumor_bam=$3\n    docm_vcf=$4\n    interval_list=$5\n    # Chaning the interval_list to a new docm_interval_list that spans the docm regions by 200bp\n    cat $interval_list | grep '^@' > docm.interval_list # Extracting the header from the interval_list\n    zcat $docm_vcf | grep ^chr | awk '{FS = \"\\t\";OFS = \"\\t\";print $1,$2-100,$2+100,\"+\",$1\"_\"$2-100\"_\"$2+100}' >> docm.interval_list # Extracting the docm regions with a 100bp flanking region on both directions\n    /gatk/gatk HaplotypeCaller --java-options \"-Xmx8g\" -R $reference -I $normal_bam -I $tumor_bam --alleles $docm_vcf -L docm.interval_list --genotyping-mode GENOTYPE_GIVEN_ALLELES -O docm_raw_variants.vcf\nelse # If normal_bam is not passed\n    reference=$1\n    tumor_bam=$2\n    docm_vcf=$3\n    interval_list=$4\n    # Chaning the interval_list to a new docm_interval_list that spans the docm regions by 200bp\n    cat $interval_list | grep '^@' > docm.interval_list # Extracting the header from the interval_list\n    zcat $docm_vcf | grep ^chr | awk '{FS = \"\\t\";OFS = \"\\t\";print $1,$2-100,$2+100,\"+\",$1\"_\"$2-100\"_\"$2+100}' >> docm.interval_list # Extracting the docm regions with a 100bp flanking region on both directions\n    /gatk/gatk HaplotypeCaller --java-options \"-Xmx8g\" -R $reference -I $tumor_bam --alleles $docm_vcf -L docm.interval_list --genotyping-mode GENOTYPE_GIVEN_ALLELES -O docm_raw_variants.vcf\nfi\n"
                        }
                    ]
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 3
                    },
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#docm_gatk_haplotype_caller.cwl/bam"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "secondaryFiles": [
                        ".tbi"
                    ],
                    "id": "#docm_gatk_haplotype_caller.cwl/docm_vcf"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5
                    },
                    "id": "#docm_gatk_haplotype_caller.cwl/interval_list"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 2
                    },
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#docm_gatk_haplotype_caller.cwl/normal_bam"
                },
                {
                    "type": [
                        "string",
                        "File"
                    ],
                    "secondaryFiles": [
                        ".fai",
                        "^.dict"
                    ],
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#docm_gatk_haplotype_caller.cwl/reference"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "docm_raw_variants.vcf"
                    },
                    "id": "#docm_gatk_haplotype_caller.cwl/docm_raw_variants"
                }
            ],
            "id": "#docm_gatk_haplotype_caller.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "Filter variants from the DoCM detector",
            "baseCommand": [
                "/usr/bin/perl",
                "docm_filter.pl"
            ],
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "mgibio/cle:v1.4.2"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 4000
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entryname": "docm_filter.pl",
                            "entry": "#!/usr/bin/perl\n\nuse strict;\nuse warnings;\n\nuse feature qw(say);\n\ndie(\"Wrong number of arguments. Provide docm_vcf, normal_cram, tumor_cram, output_vcf_file, set_filter_flag\") unless scalar(@ARGV) == 5;\nmy ($docm_vcf, $normal_cram, $tumor_cram, $output_vcf_file, $set_filter_flag) = @ARGV;\n\nmy $samtools = '/opt/samtools/bin/samtools';\nmy $normal_header_str = `$samtools view -H $normal_cram`;\nmy $tumor_header_str  = `$samtools view -H $tumor_cram`;\n\nmy ($normal_name) = $normal_header_str =~ /SM:([ -~]+)/;\nmy ($tumor_name)  = $tumor_header_str =~ /SM:([ -~]+)/;\n\nunless ($normal_name and $tumor_name) {\n    die \"Failed to get normal_name: $normal_name from $normal_cram AND tumor_name: $tumor_name from $tumor_cram\";\n}\n\nmy $docm_vcf_fh;\nif($docm_vcf =~ /.gz$/){\n    open($docm_vcf_fh, \"gunzip -c $docm_vcf |\") or die(\"couldn't open $docm_vcf to read\");\n} else {\n    open($docm_vcf_fh, $docm_vcf) or die(\"couldn't open $docm_vcf to read\");\n}\nopen(my $docm_out_fh, \">\", \"$output_vcf_file\") or die(\"couldn't open $output_vcf_file for write\");\n\nmy ($normal_index, $tumor_index);\n\nwhile (<$docm_vcf_fh>) {\n    chomp;\n    if (/^##/) {\n        say $docm_out_fh $_;\n    }\n    elsif (/^#CHROM/) {\n        if ($set_filter_flag) {\n            say $docm_out_fh '##FILTER=<ID=DOCM_ONLY,Description=\"ignore Docm variants\">';\n        }\n        my @columns = split /\\t/, $_;\n        my %index = (\n            $columns[9]  => 9,\n            $columns[10] => 10,\n        );\n        ($normal_index, $tumor_index) = map{$index{$_}}($normal_name, $tumor_name);\n        unless ($normal_index and $tumor_index) {\n            die \"Failed to get normal_index: $normal_index for $normal_name AND tumor_index: $tumor_index for $tumor_name\";\n        }\n        $columns[9]  = $normal_name;\n        $columns[10] = $tumor_name;\n        my $header = join \"\\t\", @columns;\n        say $docm_out_fh $header;\n    }\n    else {\n        my @columns = split /\\t/, $_;\n        my @tumor_info = split /:/, $columns[$tumor_index];\n        my ($AD, $DP) = ($tumor_info[1], $tumor_info[2]);\n        next unless $AD;\n        my @AD = split /,/, $AD;\n        shift @AD; #the first one is ref count\n        \n        for my $ad (@AD) {\n            if ($ad > 5 and $ad/$DP > 0.01) {\n                my ($normal_col, $tumor_col) = map{$columns[$_]}($normal_index, $tumor_index);\n                $columns[9]  = $normal_col;\n                $columns[10] = $tumor_col;\n                if ($set_filter_flag) {\n                    $columns[6] = 'DOCM_ONLY';\n                }\n                else {\n                    $columns[6] = '.';\n                }\n                my $new_line = join \"\\t\", @columns;\n                say $docm_out_fh $new_line;\n                last;\n            }\n        }\n    }\n}\n\nclose($docm_vcf_fh);\nclose($docm_out_fh);\n"
                        }
                    ]
                }
            ],
            "arguments": [
                "$(runtime.outdir)/docm_filtered_variants.vcf"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -4
                    },
                    "id": "#filter_vcf_docm.cwl/docm_raw_variants"
                },
                {
                    "type": "boolean",
                    "inputBinding": {
                        "position": 1,
                        "valueFrom": "${\n  if(inputs.filter_docm_variants){\n    return \"1\";\n  } else {\n    return \"0\";\n  }\n}\n"
                    },
                    "id": "#filter_vcf_docm.cwl/filter_docm_variants"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -3
                    },
                    "id": "#filter_vcf_docm.cwl/normal_bam"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": -2
                    },
                    "id": "#filter_vcf_docm.cwl/tumor_bam"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "docm_filtered_variants.vcf"
                    },
                    "id": "#filter_vcf_docm.cwl/docm_filtered_variants"
                }
            ],
            "id": "#filter_vcf_docm.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "vcf index",
            "arguments": [
                "cp",
                "$(inputs.vcf.path)",
                "$(runtime.outdir)/$(inputs.vcf.basename)",
                {
                    "valueFrom": " && ",
                    "shellQuote": false
                },
                "/usr/local/bin/tabix",
                "-p",
                "vcf"
            ],
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/samtools:1.11--h6270b1f_0"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 4000
                },
                {
                    "class": "StepInputExpressionRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "valueFrom": "$(self.basename)",
                        "position": 1
                    },
                    "id": "#index_vcf.cwl/vcf"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".tbi"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.vcf.basename)"
                    },
                    "id": "#index_vcf.cwl/indexed_vcf"
                }
            ],
            "id": "#index_vcf.cwl"
        },
        {
            "class": "CommandLineTool",
            "label": "run vt decompose",
            "baseCommand": [
                "vt",
                "decompose"
            ],
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/vt:0.57721--hf74b74d_1"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 4000
                }
            ],
            "arguments": [
                "-s",
                "-o",
                {
                    "valueFrom": "$(runtime.outdir)/decomposed.vcf.gz"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "secondaryFiles": [
                        ".tbi"
                    ],
                    "id": "#vt_decompose.cwl/vcf"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "decomposed.vcf.gz"
                    },
                    "id": "#vt_decompose.cwl/decomposed_vcf"
                }
            ],
            "id": "#vt_decompose.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
